// EXPERIMENTAL
// NOTE: this filter ignores the input image
public class LineGenerator2 implements PixelBufferFilter, ParameterizedFilter {
	private final IntFilterParam rParam =
		new IntFilterParam("R", Integer.MIN_VALUE, Integer.MAX_VALUE, 0);
		
	private final IntFilterParam thetaDegParam =
		new IntFilterParam("Theta", 0, 179, 30);
	
	private final FilterParam[] params = new FilterParam[] {
		rParam,
		thetaDegParam
	};
	
	// ### FILTER ###
	
	private final static int BKGND_PIXEL = Pixels.BLACK_ARGB_PIXEL;
	private final static int LINE_PIXEL = Pixels.WHITE_ARGB_PIXEL;
	
	/* First-time initialized */
	private static double[] sinTable = null;	// indexed by degrees; of length 360
	private static double[] cosTable = null;	// indexed by degrees; of length 360
	
	/* Reinitialized upon change of image size */
	private int lastWidth = -1;
	private int lastHeight = -1;
	private int originX = -1;
	private int originY = -1;
	private int maxLineLen = -1;
	
	public void filter(int width, int height, int[] pixels) {
		int thetaDeg = thetaDegParam.getValue();
		int r = rParam.getValue();
		
		int numPixels = pixels.length;
		
		/* First-time initialization */
		if (this.sinTable == null) {
			/* Compute sinTable and cosTable */
			this.sinTable = new double[360];
			this.cosTable = new double[360];
			for (int deg=0; deg<360; deg++) {
				double rad = deg*StrictMath.PI/180.0;
				this.sinTable[deg] = StrictMath.sin(rad);
				this.cosTable[deg] = StrictMath.cos(rad);
			}
		}
		
		/* Initialization upon change of image size */
		if ((width != lastWidth) || (height != lastHeight)) {
			this.originX = width/2;
			this.originY = height/2;
			
			this.maxLineLen = (((int) StrictMath.sqrt(width*width + height*height)) + 1)/2 + 1;		// +1's: compensate for roundoff error
			
			this.lastWidth = width;
			this.lastHeight = height;
		}
		
		/* Clear output image to white */
		for (int i=0; i<numPixels; i++) {
			pixels[i] = BKGND_PIXEL;
		}
		
		/* Iterate over points in line, painting each pixel */
		int phiDeg = thetaDeg+90;
		double rCosThetaDeg = r*cosTable[thetaDeg];	// r*cos(thetaDeg)
		double rSinThetaDeg = r*sinTable[thetaDeg];	// r*sin(thetaDeg)
		for (int pos = -maxLineLen; pos<=maxLineLen; pos++) {
			double posCosPhiDeg = pos*cosTable[phiDeg];	// pos*cos(phiDeg)
			double posSinPhiDeg = pos*sinTable[phiDeg];	// pos*sin(phiDeg)
			
			int x = originX + (int) (rCosThetaDeg + posCosPhiDeg);	// originX + r*cos(thetaDeg) + pos*cos(phiDeg)
			int y = originY + (int) (rSinThetaDeg + posSinPhiDeg);	// originY + r*sin(thetaDeg) + pos*sin(phiDeg)
			
			if ((x >= 0) && (x < width) && (y >= 0) && (y < height)) {	// bound check
				pixels[y*width + x] = LINE_PIXEL;
			}
		}
	}
	
	public String toString() {
		return "Line Generator #2";
	}
	
	// ### PARAMETERIZED FILTER ###
	
	public FilterParam[] getParams() {
		return params;
	}
}