public class LocalMaximaHilighter implements PixelBufferFilter2, ParameterizedFilter {
	public final IntFilterParam windowSizeParam =
		new IntFilterParam("Window Size", 1, Integer.MAX_VALUE, 10);
	
	private final FilterParam[] params = new FilterParam[] {
		windowSizeParam
	};
	
	// ### FILTER ###
	
	public void filter(int width, int height, int[] inPixels, int[] outPixels) {
		/* Convert input image from ARGB to grays */
		Pixels.ARGBPixelsToGrayPixels(inPixels, inPixels);
		
		/* Filter the gray pixels */
		filterGrays(width, height, inPixels, outPixels);
		
		/* Convert output image from grays to ARGB */
		Pixels.grayPixelsToARGBPixels(outPixels, outPixels);
	}
	
	public void filterGrays(int width, int height, int[] inGrayPixels, int[] outGrayPixels) {
		int windowSize = windowSizeParam.getValue();
		
		/* Copy input image to output image */
		System.arraycopy(inGrayPixels, 0, outGrayPixels, 0, inGrayPixels.length);
		
		/* Slide window over the input image, setting non-local-maximal pixels to black */
		int limitWindY = height-windowSize;
		int limitWindX = width-windowSize;
		for (int windY=0, windOff=0;
		     windY<limitWindY;
		     windY++, windOff+=windowSize)
		{
			for (int windX=0;
			     windX<limitWindX;
			     windX++, windOff++)
			{
				/* Assess pixels in the current window, finding the one with the maximum value */
				int maxGrayPixel = 0;
				{
					int off = windOff;
					for (int deltaY=0; deltaY<windowSize; deltaY++, off+=width-windowSize) {
						for (int deltaX=0; deltaX<windowSize; deltaX++, off++) {
							int curGrayPixel = inGrayPixels[off];
							if (curGrayPixel > maxGrayPixel) {
								maxGrayPixel = curGrayPixel;
							}
						}
					}
				}
				
				/* Set all non-maximal pixels in the current window to black */
				{
					int off = windOff;
					for (int deltaY=0; deltaY<windowSize; deltaY++, off+=width-windowSize) {
						for (int deltaX=0; deltaX<windowSize; deltaX++, off++) {
							int curGrayPixel = inGrayPixels[off];
							if (curGrayPixel != maxGrayPixel) {
								outGrayPixels[off] = Pixels.BLACK_GRAY_PIXEL;
							}
						}
					}
				}
			}
		}
	}
	
	public String toString() {
		return "Local Maxima Hilighter";
	}
	
	// ### PARAMETERIZED FILTER ###
	
	public FilterParam[] getParams() {
		return params;
	}
}