/*
 * Test images for calibration:
 * - sac_raw_1.png
 * - sac_raw_2.png
 * - sac_raw_3.png
 * - sac_raw_4.png
 */

// EXPERIMENTAL
public class WhiteHilighterFilter implements PixelFilter, ParameterizedFilter {
	/* lower values look for cleaner (less dirty) white */
	// <=40
	private final IntFilterParam whitePixelSaturationThresholdParam =
		new IntFilterParam("White Pixel Saturation - Threshold (Low)", 0, 255, 40);			
	
	/* higher values look for brighter white */
	// NOT CALIBRATED
	private final IntFilterParam whitePixelBrightnessThresholdParam =
		new IntFilterParam("White Pixel Brightness - Threshold (High)", 0, 255, 40);
	
	private final FilterParam[] params = new FilterParam[] {
		whitePixelSaturationThresholdParam,
		whitePixelBrightnessThresholdParam
	};
	
	// ### FILTER ###
	
	public int filter(int pixel) {
		int AHSBPixel = Pixels.ARGBToAHSB(pixel);
		
		/*
		int grayness = 255 - Pixels.getSaturation(AHSBPixel);
		int grayAmount = Pixels.getBrightness(AHSBPixel);
		
		int whiteness = (grayness + grayAmount)/2;		// might want to also try a geometric or harmonic mean
		return Pixels.grayToARGB(whiteness);
		*/
		
		return isWhitePixel(Pixels.getSaturation(AHSBPixel), Pixels.getBrightness(AHSBPixel))
			? Pixels.WHITE_ARGB_PIXEL
			: Pixels.BLACK_ARGB_PIXEL;
	}
	
	// NOTE: uses the same algorithm as BarrelBlobFinder for identifying white pixels
	public boolean isWhitePixel(int saturation, int brightness) {
		return
			(saturation <= whitePixelSaturationThresholdParam.getValue()) && 	// ensure "colorfulness" is sufficiently low
			(brightness >= whitePixelBrightnessThresholdParam.getValue());		// ensure "brightness" is sufficiently high
	}
	
	public String toString() {
		return "White Hilighter";
	}
	
	// ### PARAMETERIZED FILTER ###
	
	public FilterParam[] getParams() {
		return params;
	}
}